PROGRAM RS232;        (* Seriel kommunikationsprogram *)
(*--------------------------------------------------------------------------*)
(* Programmr:   Bjrk Busch          10/5 - 1989                           *)
(*                                                                          *)
(* Programmet anvendes til at illustrere den serielle kommunikation.        *)
(* Der er mulighed for svel terminal som fil overfrsel.                   *)
(*                                                                          *)
(* Programmet kan startes med angivelse af inputport og outputport          *)
(* eks.:      RS232 2 2        input COM2 output ligeledes COM2             *)
(*--------------------------------------------------------------------------*)

Uses CRT,DOS;


Const
   Paritettab: array(.0..3.) of string(.5.) = ('','Ingen','Ulige','Lige');
   ON  = True;
   OFF = False;
   ONBITTAB:  Array(.0..7.) of byte = ($01,$02,$04,$08,$10,$20,$40,$80);
   OFFBITTAB: Array(.0..7.) of byte = ($FE,$FD,$FB,$F7,$EF,$DF,$BF,$7F);
   NUL = #0;
   CTRL_C = #3;
   CtrlF1 = '^';
   CtrlF3 = '`';
   EOT = #4;
   BS  = #9;
   BEL = #7;
   LF  = #10;
   CR  = #13;
   DLE = #16;
   CAN = #24;
   ESC = #27;

   (* Liniekontrolregister signaler               *)
   LkOrdl0   = 0;      (* Ord (tegn)    00=5, 01=6  *)
   LkOrdl1   = 1;      (*   antal bit   10=7, 11=8  *)
   LkStop    = 2;      (* antal stopbits  0=1, 1=2  *)
   LkPktrl   = 3;      (* paritets kontrol   1=ON   *)
   LkParit   = 4;      (* paritet   0=ulige, 1=lige *)
   LkStikP   = 5;      (* "Stick" paritet           *)
   LkBreak   = 6;      (* Set break (0=tilad output *)
   LkDLAB    = 7;      (* Divisor Latch Access Bit  *)

   (* Modemkontrolregister signaler               *)
   MkDTR     = 0;      (* Data Terminal Ready       *)
   MkRTS     = 1;      (* Request To Send           *)
   MkOut1    = 2;      (* Out1                      *)
   MkOut2    = 3;      (* Out2                      *)
   MkLoop    = 4;      (* Loop                      *)

   (* Liniestatus signaler                        *)
   LsData    = 0;      (* data parat (input)        *)
   LsOverfl  = 1;      (* overlbsfejl              *)
   LsParErr  = 2;      (* paritetsfejl              *)
   LsOrdErr  = 3;      (* fejl i ord eller stopbit  *)
   LsBreak   = 4;      (* Break interrupt           *)
   LsTHRemty = 5;      (* Transmit holding reg. tom *)
   LsTSRemty = 6;      (* Transmit Shift reg. tom   *)

   (* Modemstatus signaler                        *)
   MsDCTS    = 0;      (* CTS ndret  (delta)       *)
   MsDDSR    = 1;      (* DSR ndret  (delta)       *)
   MsDRNG    = 2;      (* RNG ndret  (delta)       *)
   MsDDCD    = 3;      (* DCD ndret  (delta)       *)
   MsCTS     = 4;      (* Clear To Send             *)
   MsDSR     = 5;      (* Data Set Ready            *)
   MsRNG     = 6;      (* Ring Indikator            *)
   MsDCD     = 7;      (* Data Carrier Detect       *)
Type
   SkrmSideType = Array(.1..4000.) of char;
   ComPortType = RECORD
      LineOut:      integer;    (* transmitter holding register - DLA = 0 *)
      LineIn:       integer;    (* reciever-buffer register     - DLA = 0 *)
      DivisorLsb:   integer;    (* Divisorlatch register (LSB)  - DLA = 1 *)
      DivisorMsb:   integer;    (* Divisorlatch register (MSB)  - DLA = 1 *)
      IntEna:       integer;    (* Interupt enable register     *)
      IntID:        integer;    (* Interupt ID register         *)
      LineCtrl:     integer;    (* Line-kontrol register        *)
      ModemCtrl:    integer;    (* Modem-kontrol register       *)
      LineSta:      integer;    (* Line-status register         *)
      ModemSta:     integer;    (* Modem-status register        *)
      Baud:         integer;    (* Baud-rate                    *)
      Paritet:      integer;    (* 1=ingen, 2= ulige, 3=lige    *)
      StopBit:      integer;    (* antal stopbit                *)
      Word:         integer;    (* antal bit i tegn             *)
   END;
   ComPorteType = Array(.1..4.) of ComPortType;

   ModemKontrolBitType = RECORD
      DTR:     Boolean;  (* Data Terminal Ready       *)
      RTS:     Boolean;  (* Request To Send           *)
      Out1:    Boolean;  (* Out1                      *)
      Out2:    Boolean;  (* Out2                      *)
      Loop:    Boolean;  (* Loop                      *)
   END;
   LineStatusBitType = RECORD
      Data:    Boolean;  (* data parat (input)        *)
      Overfl:  Boolean;  (* overlbsfejl              *)
      ParErr:  Boolean;  (* paritetsfejl              *)
      OrdErr:  Boolean;  (* fejl i ord eller stopbit  *)
      Break:   Boolean;  (* Break interrupt           *)
      THRemty: Boolean;  (* Transmit holding reg. tom *)
      TSRemty: Boolean;  (* Transmit Shift reg. tom   *)
   END;

   ModemstatusBitType = RECORD
      DCTS:    Boolean;  (* CTS ndret  (delta)       *)
      DDSR:    Boolean;  (* DSR ndret  (delta)       *)
      DRNG:    Boolean;  (* RNG ndret  (delta)       *)
      DDCD:    Boolean;  (* DCD ndret  (delta)       *)
      CTS:     Boolean;  (* Clear To Send             *)
      DSR:     Boolean;  (* Data Set Ready            *)
      RNG:     Boolean;  (* Ring Indikator            *)
      DCD:     Boolean;  (* Data Carrier Detect       *)
   END;

Var
   Com: ComPorteType;
   ConNr: integer;

   Tegn:        Char;
   Status:      Char;
   StatusC:     Char;
   SR:          Char;
   Tidsfakt:    Integer;
   SkrmBufAdr:  Word;

Function GetSkrmBufAdr: Word;
var
   Regs:        Registers;
   i:           integer;
   SIEMENS:     boolean;
const
   navn: array(.1..7.) of char = 'SIEMENS';
begin
   Regs.Ax := $0F00;
   Intr($10,Regs);        (* hent video mode i Al og page i Bh *)
   siemens := true;
   for i := 1 to 7 do
      if chr(mem(.$FF00:$0BF0+i.)) <> navn(.i.) then SIEMENS := false;
   if SIEMENS then
      GetSkrmBufAdr := $F000
   else begin
      Regs.Ax := $0F00;
      Intr($10,Regs);    (* hent video mode *)
      if Regs.Al = 7 then GetSkrmBufAdr := $B000  { Mono }
      else                GetSkrmBufAdr := $B800; { color }
   end;

end;  (* slut p SkrmBufAdr *)
(*--------------------------------------------------------------------------*)
Type
   hexud = string(.2.);
Function Hex (tegn: char): hexud;
Const
   hextab: array(.0..15.) of char = ('0','1','2','3','4','5','6','7',
                                     '8','9','A','B','C','D','E','F');
begin
    hex := hextab(.ord(tegn) DIV 16.)+hextab(.ord(tegn) MOD 16.);
end; (* hex *)
(*--------------------------------------------------------------------------*)

Procedure Getxy(Var x,y: integer);

Var
   Regs: Registers;
begin
   Regs.Ah := 3;     (* get cursor pos *)
   Regs.Bh := 0;     (* side 0         *)
   IntR($10,Regs);
   x := Regs.Dl+1;
   y := Regs.Dh+1;
end;

Procedure Scroll(x1,y1,x2,y2: integer);
Var
   Regs: Registers;
begin
   Regs.Ah := 6;      (* scroll up  *)
   Regs.Al := 1;      (* 1 linie    *)
   Regs.Bh := 7;      (* att=normal *)
   Regs.Cl := x1-1;
   Regs.Ch := y1-1;
   Regs.Dl := x2-1;
   Regs.Dh := Y2-1;
   IntR($10,Regs);
end;

Procedure DosSkrivTegn(Tegn: Char);
Var
   Regs: Registers;
begin
   Regs.Ah := 2;
   Regs.Dl := Ord(tegn);
   IntR($21,Regs);
end;

Procedure SkrivTegn(Tegn: Char);
begin
   Case tegn of
      BEL: begin
              TextColor(Black); TextBackground(White);
              Write('G');
              TextColor(White); TextBackground(Black);
           end;
     #27: begin
              TextColor(Black); TextBackground(White);
              Write('[');
              TextColor(White); TextBackground(Black);
           end;
      ELSE Write(Tegn);
   end;
end;

Procedure InitPort(ComNr: integer);

Var
   Regs: Registers;
   Parm: byte;
   Divisor: word;
begin
   (* Paritet: 0=none, 1=odd, 2=even *)

   Case Com(.ComNr.).Word of
      5:   Parm := $00;
      6:   Parm := $01;
      7:   Parm := $02;
      else begin
           Parm := $03;
           Com(.ComNr.).Word := 8;
      end;
   end;
   Case Com(.ComNr.).Stopbit of
      1:;
      2:   Parm := Parm OR $04;
      else Com(.ComNr.).StopBit := 1;
   end;
   Case Com(.ComNr.).Paritet of
      1:;
      2:   Parm := Parm OR $08;
      3:   Parm := Parm OR $18;
      else Com(.ComNr.).Paritet := 1;
   end;
   Divisor := Round(1.152E5 / Com(.ComNr.).Baud);
   Port(.Com(.ComNr.).LineCtrl.)   := $80;     (* st DLAB=1  -  ben latch *)
   Port(.Com(.ComNr.).DivisorLsb.) := Lo(Divisor);
   Port(.Com(.ComNr.).DivisorMsb.) := Hi(Divisor);
   Port(.Com(.ComNr.).LineCtrl.)   := Parm;    (* st parm - luk latch *)
   Port(.Com(.ComNr.).IntEna.)     := $00;     (* Ingen interupt *)

end;

Procedure InitRS232(RS: char; Var ComNrI, ComNrO: integer);
Const
   ParitetTab: Array(.1..3.) of char = ('I','U','L');
Var
   Tegn: Char;
   ComNr: integer;
   SkrmP:   ^SkrmSideType;
   GemSkrm:  SkrmSideType;
   Stal:     string(.6.);
   Tal,Test: integer;
   x,y: integer;
   gx,gy: integer;
begin
   GetXY(gx,gy);
   SkrmP := Ptr(SkrmBufAdr,0);
   GemSkrm := SkrmP^;
   Repeat
      ClrScr;
      Case RS of
         'r','R','x','X':
               begin
                  WriteLN('Justering af port for modtagelse');
                  ComNr := ComNrI;
               end;
         's','S':
               begin
                  WriteLN('Justering af port for afsendelse');
                  ComNr := ComNrO;
               end;
         else  begin
                  WriteLN('Justering af port');
                  ComNr := ComNrI;
               end;
      end;
      WriteLN;
      WriteLN('Port:   1=COM1, 2= COM2, 3= COM3, 4= COM4');
      Write('Indtast portnr NU: ',ComNr,'    ');
      Repeat
         Tegn := ReadKey;
         Write(Tegn);
         if NOT (Tegn IN (.'1','2','3','4',CR.)) then Write(BEL,BS);
      until (Tegn IN (.'1','2','3','4',CR.));
      Case Tegn of
         '2': ComNr := 2;
         '3': ComNr := 3;
         '4': ComNr := 4;
         else ComNr := 1;
      end;
      WriteLN;

      WriteLN('Baud:   110, 150, 300, 600, 1200, 2400, 4800, 9600, 19200');
      Write('Indtast Baud-hastighed   - Nu: ',Com(.ComNr.).Baud,'   ');
      GetXY(x,y);
      Tal := Com(.ComNr.).Baud;
      Repeat
         Gotoxy(x,y);
         ReadLN(Stal);
         if Stal <> '' then begin
            Val(Stal,Tal,Test);
            Gotoxy(x,y);
            if (Tal < 2) OR (Tal > 19200) then Tal := 110;
            Tal := Round(1.152E5 / Round(1.152E5 / Tal));
            Write(Tal,'      ');
         end
         else Test := 0;
      until test = 0;
      Com(.ComNr.).Baud := Tal;
      WriteLN;

      WriteLN;
      WriteLN('Parity: I=ingen, U=ulige, L=lige ');
      Write('Indtast Paritet   - Nu: ',ParitetTab(.Com(.ComNr.).Paritet.),'   ');
      Repeat
         Tegn := ReadKey;
         if Tegn = 'i' then Tegn := 'I';
         if Tegn = 'u' then Tegn := 'U';
         if Tegn = 'l' then Tegn := 'L';
         Write(Tegn);
         if NOT (Tegn IN (.'I','U','L',CR.)) then Write(BEL,BS);
      until (Tegn IN (.'I','U','L',CR.));
      if Tegn = 'I' then      Com(.ComNr.).Paritet := 1
      else if Tegn = 'U' then Com(.ComNr.).Paritet := 2
      else if Tegn = 'L' then Com(.ComNr.).Paritet := 3;
      WriteLN;

      WriteLN;
      WriteLN('StopBit: 1, 2 ');
      Write('Indtast Stopbits   - Nu: ',Com(.ComNr.).StopBit,'   ');
      Repeat
         Tegn := ReadKey;
         Write(Tegn);
         if NOT (Tegn IN (.'1','2',CR.)) then Write(BEL,BS);
      until (Tegn IN (.'1','2',CR.));
      if Tegn = '1' then      Com(.ComNr.).StopBit := 1
      else if Tegn = '2' then Com(.ComNr.).StopBit := 2;
      WriteLN;

      WriteLN;
      WriteLN('Word:   5, 6, 7, 8 ');
      Write('Indtast bit i tegn   - Nu: ',Com(.ComNr.).Word,'   ');
      Repeat
         Tegn := ReadKey;
         Write(Tegn);
         if NOT (Tegn IN (.'1'..'8',CR.)) then Write(BEL,BS);
      until (Tegn IN (.'5'..'8',CR.));
      if Tegn = '5' then      Com(.ComNr.).Word := 5
      else if Tegn = '6' then Com(.ComNr.).Word := 6
      else if Tegn = '7' then Com(.ComNr.).Word := 7
      else if Tegn = '8' then Com(.ComNr.).Word := 8;
      WriteLN;

      InitPort(ComNr);

      Case RS of
         'x','X': begin RS := 'S'; ComNrI := ComNr; end;
         's','S': begin RS := ' '; ComNrO := ComNr; end;
         else     begin RS := ' '; ComNrI := ComNr; end;
      end;
   until RS = ' ';

   SkrmP^ := GemSkrm;
   GotoXY(gx,gy);
end;

Procedure OpsaetPortAdr;
Var
   ComNr:       integer;
   ComPortAdr:  ^integer;
begin
   For ComNr := 1 to 4 do begin
      ComPortAdr := Ptr($40,(ComNr-1)*2);
      Com(.ComNr.).LineOut    := ComPortAdr^;
      if Com(.ComNr.).LineOut = 0 then case ComNr of
         1:  Com(.ComNr.).LineOut := $3F8;
         2:  Com(.ComNr.).LineOut := $2F8;
         3:  Com(.ComNr.).LineOut := $3E8;
         4:  Com(.ComNr.).LineOut := $2E8;
      end;
      Com(.ComNr.).LineIn     := Com(.ComNr.).LineOut + 0;
      Com(.ComNr.).DivisorLSB := Com(.ComNr.).LineOut + 0;
      Com(.ComNr.).DivisorMSB := Com(.ComNr.).LineOut + 1;
      Com(.ComNr.).IntEna     := Com(.ComNr.).LineOut + 1;
      Com(.ComNr.).IntID      := Com(.ComNr.).LineOut + 2;
      Com(.ComNr.).LineCtrl   := Com(.ComNr.).LineOut + 3;
      Com(.ComNr.).ModemCtrl  := Com(.ComNr.).LineOut + 4;
      Com(.ComNr.).LineSta    := Com(.ComNr.).LineOut + 5;
      Com(.ComNr.).ModemSta   := Com(.ComNr.).LineOut + 6;
      Com(.ComNr.).Baud       := 300;
      Com(.ComNr.).Paritet    := 1;
      Com(.ComNr.).Stopbit    := 0;
      Com(.ComNr.).Word       := 8;
   end;
{
   WriteLN(Hex(Chr(hi(Com(.1.).LineOut))),Hex(Chr(lo(Com(.1.).LineOut))) );
   WriteLN(Hex(Chr(hi(Com(.2.).LineOut))),Hex(Chr(lo(Com(.2.).LineOut))) );
   WriteLN(Hex(Chr(hi(Com(.3.).LineOut))),Hex(Chr(lo(Com(.3.).LineOut))) );
   WriteLN(Hex(Chr(hi(Com(.4.).LineOut))),Hex(Chr(lo(Com(.4.).LineOut))) );
   ReadLN;
}
end;

Procedure xGetModemKontrol(ComNr: integer; ModemKontrol: ModemKontrolBitType);
Var
   Portnr: integer;
   IByte:  Byte;
Begin
   Portnr := Com(.ComNr.).ModemCtrl;
   IByte := Port(.Portnr.);
   ModemKontrol.DTR  := ( (IByte AND ONBITTAB(.MkDTR.)) <> 0 );
   ModemKontrol.RTS  := ( (IByte AND ONBITTAB(.MkRTS.)) <> 0 );
   ModemKontrol.Out1 := ( (IByte AND ONBITTAB(.MkOut1.)) <> 0 );
   ModemKontrol.Out2 := ( (IByte AND ONBITTAB(.MkOut2.)) <> 0 );
   ModemKontrol.Loop := ( (IByte AND ONBITTAB(.MkLoop.)) <> 0 );
end;

Procedure xPutModemKontrol(ComNr: integer; ModemKontrol: ModemKontrolBitType);
Var
   Portnr: integer;
   OByte:  Byte;
Begin
   Portnr := Com(.ComNr.).ModemCtrl;
   OByte  := Port(.Portnr.);
   if ModemKontrol.DTR  then OByte := (OByte AND ONBITTAB(.MkDTR.))
   else                      OByte := (Obyte AND OFFBITTAB(.MkDTR.));
   if ModemKontrol.RTS  then OByte := (OByte AND ONBITTAB(.MkRTS.))
   else                      OByte := (Obyte AND OFFBITTAB(.MkRTS.));
   if ModemKontrol.Out1 then OByte := (OByte AND ONBITTAB(.MkOut1.))
   else                      OByte := (Obyte AND OFFBITTAB(.MkOut1.));
   if ModemKontrol.Out2 then OByte := (OByte AND ONBITTAB(.MkOut1.))
   else                      OByte := (Obyte AND OFFBITTAB(.MkOut2.));
   if ModemKontrol.Loop then OByte := (OByte AND ONBITTAB(.MkLoop.))
   else                      OByte := (Obyte AND OFFBITTAB(.MkLoop.));
   Port(.Portnr.) := Obyte;
end;

Procedure xPutLineStatus(ComNr: integer; LineStatus: LineStatusBitType);
Var
   Portnr: integer;
   OByte:  Byte;
Begin
   Portnr := Com(.ComNr.).ModemCtrl;
   OByte  := Port(.Portnr.);
   if LineStatus.Data    then OByte := (OByte AND ONBITTAB(.LsData.))
   else                       OByte := (Obyte AND OFFBITTAB(.LsData.));
   if LineStatus.Overfl  then OByte := (OByte AND ONBITTAB(.LsOverfl.))
   else                       OByte := (Obyte AND OFFBITTAB(.LsOverfl.));
   if LineStatus.ParErr  then OByte := (OByte AND ONBITTAB(.LsParErr.))
   else                       OByte := (Obyte AND OFFBITTAB(.LsParErr.));
   if LineStatus.OrdErr  then OByte := (OByte AND ONBITTAB(.LsOrdErr.))
   else                       OByte := (Obyte AND OFFBITTAB(.LsOrdErr.));
   if LineStatus.Break   then OByte := (OByte AND ONBITTAB(.LsBreak.))
   else                       OByte := (Obyte AND OFFBITTAB(.LsBreak.));
   if LineStatus.THRemty then OByte := (OByte AND ONBITTAB(.LsTHRemty.))
   else                       OByte := (Obyte AND OFFBITTAB(.LsTHRemty.));

   Port(.Portnr.) := Obyte;
end;

Function ModemKontrol(ComNr,bitnr: integer): boolean;
Var Portnr: integer;
Begin
   Portnr := Com(.ComNr.).ModemCtrl;
   ModemKontrol := (Port(.Portnr.) AND ONBITTAB(.bitnr.)) <> 0;
end;

Function ModemStatus(ComNr,bitnr: integer): boolean;
Var Portnr: integer;
Begin
   Portnr := Com(.ComNr.).ModemSta;
   ModemStatus := (Port(.Portnr.) AND ONBITTAB(.bitnr.)) <> 0;
end;

Function LineKontrol(ComNr,bitnr: integer): boolean;
Var Portnr: integer;
Begin
   Portnr := Com(.ComNr.).LineCtrl;
   LineKontrol := (Port(.Portnr.) AND ONBITTAB(.bitnr.)) <> 0;
end;

Function LineStatus(ComNr,bitnr: integer): boolean;
Var Portnr: integer;
    Status: integer;
    ch: char;
Begin
   Portnr := Com(.ComNr.).LineSta;
   LineStatus := (Port(.Portnr.) AND ONBITTAB(.bitnr.)) <> 0;
end;

Procedure SetDTR(ComNr: integer; ON: boolean );
Var Portnr: integer;
Begin
   Portnr := Com(.ComNr.).ModemCtrl;
   if ON then Port(.Portnr.) := Port(.Portnr.) OR ONBITTAB(.MkDTR.)
   else       Port(.Portnr.) := Port(.Portnr.) AND OFFBITTAB(.MkDTR.);
end;
Procedure SetRTS(ComNr: integer; ON: boolean );
Var Portnr: integer;
Begin
   Portnr := Com(.ComNr.).ModemCtrl;
   if ON then Port(.Portnr.) := Port(.Portnr.) OR ONBITTAB(.MkRTS.)
   else       Port(.Portnr.) := Port(.Portnr.) AND OFFBITTAB(.MkRTS.);
end;
Procedure SetOut1(ComNr: integer; ON: boolean );
Var Portnr: integer;
Begin
   Portnr := Com(.ComNr.).ModemCtrl;
   if ON then Port(.Portnr.) := Port(.Portnr.) OR ONBITTAB(.MkOut1.)
   else       Port(.Portnr.) := Port(.Portnr.) AND OFFBITTAB(.MkOut1.);
end;
Procedure SetOut2(ComNr: integer; ON: boolean );
Var Portnr: integer;
Begin
   Portnr := Com(.ComNr.).ModemCtrl;
   if ON then Port(.Portnr.) := Port(.Portnr.) OR ONBITTAB(.MkOut2.)
   else       Port(.Portnr.) := Port(.Portnr.) AND OFFBITTAB(.MkOut2.);
end;
Procedure SetLoop(ComNr: integer; ON: boolean );
Var Portnr: integer;
Begin
   Portnr := Com(.ComNr.).ModemCtrl;
   if ON then Port(.Portnr.) := Port(.Portnr.) OR ONBITTAB(.MkLoop.)
   else       Port(.Portnr.) := Port(.Portnr.) AND OFFBITTAB(.MkLoop.);
end;

Function DTR(ComNr:integer) : boolean;
Begin DTR := ModemKontrol(ComNr,MkDTR); end;
Function RTS(ComNr:integer) : boolean;
Begin RTS := ModemKontrol(ComNr,MkRTS); end;

Function CTS(ComNr:integer) : boolean;
Begin CTS := ModemStatus(ComNr,MsCTS); end;
Function DSR(ComNr:integer) : boolean;
Begin DSR := ModemStatus(ComNr,MsDSR); end;
Function RNG(ComNr:integer) : boolean;
Begin RNG := ModemStatus(ComNr,MsRNG); end;
Function DCD(ComNr:integer) : boolean;
Begin DCD := ModemStatus(ComNr,MsDCD); end;

Function DataReady(ComNr:integer) : boolean;
Begin DataReady := LineStatus(ComNr,LsData); end;
Function LineOverfl(ComNr:integer) : boolean;
Begin LineOverfl := LineStatus(ComNr,LsOverfl); end;
Function ParitetErr(ComNr:integer) : boolean;
Begin ParitetErr := LineStatus(ComNr,LsParErr); end;
Function OrdErr(ComNr:integer) : boolean;
Begin OrdErr := LineStatus(ComNr,LsOrdErr); end;
Function Break(ComNr:integer) : boolean;
Begin Break := LineStatus(ComNr,LsBreak); end;
Function THRemty(ComNr:integer) : boolean;
Begin THRemty := LineStatus(ComNr,LsTHRemty); end;
Function TSRemty(ComNr:integer) : boolean;
Begin TSRemty := LineStatus(ComNr,LsTSRemty); end;



Procedure SendTegn(ComNr: integer; Tegn: Char);
Begin Port(.Com(.ComNr.).LineOut.) := ord(tegn); End;

Function ReceivedTegn(ComNr:integer) : Char;
Begin ReceivedTegn := Chr(Port(.Com(.ComNr.).LineIn.)); End;



Procedure VisSignaler(ComNrI,ComNrO:integer);
Var i: integer;
begin
   Gotoxy(01,01);
   WriteLN(Hex(Chr(hi(Com(.ComNrI.).LineOut))),Hex(Chr(lo(Com(.ComNrI.).LineOut))) );
   Gotoxy(07,01); Write('Modem Kontrol COM',ComNrI);
   Gotoxy(41,01); Write('DTR: ',DTR(ComNrI),' ');
   Gotoxy(61,01); Write('RTS: ',RTS(ComNrI),' ');
   Gotoxy(01,02); Write('Modem Status COM',ComNrI);
   Gotoxy(01,03); Write('CTS: ',CTS(ComNrI),' ');
   Gotoxy(21,03); Write('DSR: ',DSR(ComNrI),' ');
   Gotoxy(41,03); Write('RNG: ',RNG(ComNrI),' ');
   Gotoxy(61,03); Write('DCD: ',DCD(ComNrI),' ');
   Gotoxy(01,04); Write('Line Status COM',ComNrI);
   Gotoxy(01,05); Write('DataReady:  ',DataReady(ComNrI),' ');
   Gotoxy(41,05); Write('LineOverfl: ',LineOverfl(ComNrI),' ');
   Gotoxy(01,06); Write('ParitetErr: ',ParitetErr(ComNrI),' ');
   Gotoxy(41,06); Write('OrdErr:     ',OrdErr(ComNrI),' ');
   Gotoxy(01,07); Write('Break:      ',Break(ComNrI),' ');
   Gotoxy(41,07); Write('THRemty:    ',THRemty(ComNrI),' ');
   Gotoxy(41,08); Write('TSRemty:    ',TSRemty(ComNrI),' ');
   Gotoxy(01,09); for i := 1 to 80 do Write('-');

   Gotoxy(01,10);
   WriteLN(Hex(Chr(hi(Com(.ComNrO.).LineOut))),Hex(Chr(lo(Com(.ComNrO.).LineOut))) );
   Gotoxy(07,10); Write('Modem Kontrol COM',ComNrO);
   Gotoxy(41,10); Write('DTR: ',DTR(ComNrO),' ');
   Gotoxy(61,10); Write('RTS: ',RTS(ComNrO),' ');
   Gotoxy(01,11); Write('Modem Status COM',ComNrO);
   Gotoxy(01,12); Write('CTS: ',CTS(ComNrO),' ');
   Gotoxy(21,12); Write('DSR: ',DSR(ComNrO),' ');
   Gotoxy(41,12); Write('RNG: ',RNG(ComNrO),' ');
   Gotoxy(61,12); Write('DCD: ',DCD(ComNrO),' ');
   Gotoxy(01,13); Write('Line Status COM',ComNrO);
   Gotoxy(01,14); Write('DataReady:  ',DataReady(ComNrO),' ');
   Gotoxy(41,14); Write('LineOverfl: ',LineOverfl(ComNrO),' ');
   Gotoxy(01,15); Write('ParitetErr: ',ParitetErr(ComNrO),' ');
   Gotoxy(41,15); Write('OrdErr:     ',OrdErr(ComNrO),' ');
   Gotoxy(01,16); Write('Break:      ',Break(ComNrO),' ');
   Gotoxy(41,16); Write('THRemty:    ',THRemty(ComNrO),' ');
   Gotoxy(41,17); Write('TSRemty:    ',TSRemty(ComNrO),' ');
   Gotoxy(01,18); for i := 1 to 80 do Write('-');
   WriteLN;
end;

Procedure testport;
Var
   Valg:   char;
   ComNrI: integer;
   ComNrO: Integer;
   Portnr: integer;
begin
   ClrScr;
   Write('Indtast nr. p frste com-port 1,2,3,4 ');
   Repeat
      Valg := ReadKey;
   until Valg IN ['1'..'4'];
   ComNrI := Ord(Valg) - Ord('0');
   WriteLN(Valg);
   Write('Indtast nr. p anden com-port 1,2,3,4 ');
   Repeat
      Valg := ReadKey;
   until Valg IN ['1'..'4'];
   ComNrO := Ord(Valg) - Ord('0');
   WriteLN(Valg);

   Portnr := Com(.ComNrO.).ModemCtrl;

   ClrScr;
   Gotoxy(1,20);
   WriteLN('ndring af Modem kontrol koder');
   WriteLN('COM',ComNrI,'                                  COM',ComNrO);
   WriteLN('a: DTR ON   b: DTR OFF                A: DTR ON     B: DTR OFF');
   WriteLN('c: RTS ON   d: RTS OFF                C: RTS ON     D: RTS OFF');
   WriteLN;
   WriteLN('x: afslut display / ndring af status / kontrol porte');

   VisSignaler(ComNrI,ComNrO);
   Valg := ' ';
   repeat
      GotoXy(01,25);
      if KeyPressed then Valg := ReadKey;
      Case Valg of
         'a':     SetDTR(ComNrI,ON);
         'b':     SetDTR(ComNrI,OFF);
         'c':     SetRTS(ComNrI,ON);
         'd':     SetRTS(ComNrI,OFF);
         'A':     SetDTR(ComNrO,ON);
         'B':     SetDTR(ComNrO,OFF);
         'C':     SetRTS(ComNrO,ON);
         'D':     SetRTS(ComNrO,OFF);
         Else;
      End;
      VisSignaler(ComNrI,ComNrO);
   until Valg IN (.'x','X'.);
   ClrScr;
end;

Procedure TransSkrm(ComNrI,ComNrO,Sendmin,Sendmax,Recmin,Recmax: integer;
                    RetBaud: boolean);
Var
   Xi: integer;
begin
   ClrScr;
   TextColor(Black); TextBackground(White);
   Gotoxy(80,25);        Write(' ');   (* provoker skift - bund -> sort *)
   Gotoxy(01,Recmin-1);  For xi := 1 to 74 do Write('-'); Write(' COM',ComNrI,' ');
   Gotoxy(01,Recmin-1);  Write(' Modtager   ------ ',
                               Com(.ComNrI.).Baud,' Baud, ',
                               Com(.ComNrI.).Word,' Bits tegn, ',
                               Paritettab(.Com(.ComNrI.).Paritet.),' Paritet, ',
                               Com(.ComNrI.).Stopbit,' Stopbits ');
   Gotoxy(01,Sendmin-1); For xi := 1 to 74 do Write('-'); Write(' COM',ComNrO,' ');
   Gotoxy(01,Sendmin-1); Write(' Afsender   ------ ',
                               Com(.ComNrO.).Baud,' Baud, ',
                               Com(.ComNrO.).Word,' Bits tegn, ',
                               Paritettab(.Com(.ComNrO.).Paritet.),' Paritet, ',
                               Com(.ComNrO.).Stopbit,' Stopbits ');
   Gotoxy(01,25);
   if NOT Retbaud then Write(' Tast CTRL+F1 for at afbryde rutine')
   else Write(' Tast CTRL+F1 for af afbryde og CTRL+F3 for at ndre baud-rate m.m.');
   TextColor(White); TextBackground(Black);
   Gotoxy(80,24);        Write(' ');    (* blank sort tegn igen *)
end;

Procedure ReceiveSend(ComNrI,ComNrO: integer);
Const
   Sendmin = 17; SendMax = 24;
   Recmin  =  2; Recmax  = 15;
Var
   TegnI,TegnO: Char;
   Xi,Yi,Xo,Yo: Integer;
   Tegn: char;
   FunkTegn: char;
   FunkSend: byte;
begin
   TransSkrm(ComNrI,ComNrO,Sendmin,Sendmax,Recmin,Recmax,True);

   if DataReady(ComNrI) then TegnI := ReceivedTegn(ComNrI); (* tm evt. gl. *)

   Xi := 1; Yi := Recmin;
   Xo := 1; Yo := Sendmin;
   TegnO := ' ';
   Gotoxy(Xo,Yo);
   FunkSend := 0;
   FunkTegn := NUL;
   Repeat
      Gotoxy(Xo,Yo);
      if KeyPressed OR (FunkSend > 0) then if THRemty(ComNrO) then begin
         if FunkSend > 0 then begin
            if FunkSend = 2 then
               TegnO := NUL
            else begin
               TegnO    := FunkTegn;
               FunkTegn := NUL;
            end;
            FunkSend := FunkSend - 1;
         end
         else begin
            TegnO := ReadKey;
            if TegnO = NUL then begin
               FunkTegn := ReadKey;
               FunkSend := 2;
            end;
         end;
         if FunkSend <> 2 then begin
            SendTegn(ComNrO,TegnO);
            Gotoxy(Xo,Yo); SkrivTegn(TegnO);
            if TegnO = CR then SkrivTegn(LF);
            Getxy(Xo,Yo);
            if Yo > SendMax then begin
               Scroll(1,SendMin,80,SendMax);
               Xo := 1; Yo := SendMax
            end;
         end;
         if FunkTegn = CtrlF3 then begin
            InitRS232('x',ComNrI,ComNrO);
            TextColor(White); TextBackground(Black);
            FunkTegn := NUL;
            FunkSend := 0;
         end;
      end;
      While DataReady(ComNrI) do begin
         TegnI := ReceivedTegn(ComNrI);
         Gotoxy(Xi,Yi); SkrivTegn(TegnI);
         Getxy(Xi,Yi);
         if Yi > RecMax then begin
            Scroll(1,RecMin,80,RecMax);
            Xi := 1; Yi := RecMax;
         end;
      end;
      if TegnO = CR then begin
         Repeat until THRemty(ComNrO);
         TegnO := LF;
         SendTegn(ComNrO,TegnO);
      end;
   Until FunkTegn = CtrlF1;

   Gotoxy(01,25);
   TextColor(Black); TextBackground(White);
   Write(' Indtast afslutningsmde  E = End of trans., C = Cancel og retur = alm.       ');
   Repeat TegnO := ReadKey; until TegnO IN (.'E','e','C','c',CR.);
   Repeat until THRemty(ComNrO);
   Gotoxy(01,25);
   Case TegnO of
      'E','e': begin
         SendTegn(ComNrO,EOT);  (* End Of Transmision *)
         Write(' Transmision er afsluttet med EOT - tryk p en tast                           ');
      end;
      'C','c': begin
         SendTegn(ComNrO,CAN);   (* Cancel *)
         Write(BEL,' Transmision er AFBRUDT med CAN - tryk p en tast                             ');
      end;
      else begin
         Write(' Transmision er afsluttet - tryk p en tast                                   ');
      end;
   end;
   TextColor(White); TextBackground(Black);
   TegnO := ReadKey;
end;

Procedure TTYTerminalEmulering(ComNrI,ComNrO: integer);
Var
   TegnI,TegnO: Char;
   Tegn: char;
   FunkTegn: char;
   FunkSend: byte;
begin
   ClrScr;
   if DataReady(ComNrI) then TegnI := ReceivedTegn(ComNrI); (* tm evt. gl. *)
   Gotoxy(01,01);
   TextColor(Black); TextBackground(White);
   WriteLN(' TTY terminal emulering  afbryd med CTRL + F1                                   ');
   TextColor(White); TextBackground(Black);
   TegnO := ' ';
   FunkSend := 0;
   FunkTegn := NUL;
   Repeat
      if KeyPressed OR (FunkSend > 0) then if THRemty(ComNrO) then begin
         if FunkSend > 0 then begin
            if FunkSend = 2 then
               TegnO := NUL
            else begin
               TegnO    := FunkTegn;
               FunkTegn := NUL;
            end;
            FunkSend := FunkSend - 1;
         end
         else begin
            TegnO := ReadKey;
            if TegnO = NUL then begin
               FunkTegn := ReadKey;
               FunkSend := 2;
            end;
         end;
         if FunkSend <> 2 then begin
            SendTegn(ComNrO,TegnO);
         end;
         if FunkTegn = CtrlF3 then begin
            InitRS232(' ',ComNrI,ComNrO);
            TextColor(White); TextBackground(Black);
            FunkTegn := NUL;
            FunkSend := 0;
         end;
      end;
      While DataReady(ComNrI) do begin
         TegnI := ReceivedTegn(ComNrI);
         DosSkrivTegn(TegnI);
      end;
      if TegnO = CR then begin
         Repeat until THRemty(ComNrO);
         TegnO := LF;
         SendTegn(ComNrO,TegnO);
      end;
   Until FunkTegn = CtrlF1;

   Gotoxy(01,25);
   TextColor(Black); TextBackground(White);
   Write(' TTY terminal emulering er afsluttet - tryk p en tast                         ');
   TextColor(White); TextBackground(Black);
   TegnO := ReadKey;
end;

Procedure SendFile(ComNrI,ComNrO: integer);
Const
   Sendmin = 17; SendMax = 24;
   Recmin  =  2; Recmax  = 15;
Var
   TegnI,TegnO: Char;
   Xi,Yi,Xo,Yo: Integer;
   Tegn:        char;
   InFile:      File of char;
   InFileName:  string(.64.);
   IOtest:      integer;
   SpecTegn:    char;
   Stop:        boolean;
   FunkTegn: char;
begin
   ClrScr;
   Repeat
      Write('Indtast Navn p afsender fil: ');ReadLN(InFileName);
      if InFileName = '' then EXIT;
      Assign(Infile,InFileName);
      (*$I-*) Reset(InFile); (*$I+*)
      IOtest := IOresult;
      if IOtest <> 0 then begin
         WriteLN(BEL,'Fejl ved bning af fil  -  fejlkode ',Hex(Chr(lo(IOtest))));
      end;
      if IOtest = 0 then begin
         if Eof(InFile) then begin
            WriteLN(BEL,'Tomt dataset kan ikke sendes');
            IOtest := 255;
         end;
      end;
   until IOtest = 0;

   TransSkrm(ComNrI,ComNrO,Sendmin,Sendmax,Recmin,Recmax,True);

   if DataReady(ComNrI) then TegnI := ReceivedTegn(ComNrI); (* tm evt. gl. *)

   Xi := 1; Yi := Recmin;
   Xo := 1; Yo := Sendmin;

   Stop := False;
   SpecTegn := NUL;
   TegnI := ' ';
   FunkTegn := NUL;
   Gotoxy(Xo,Yo);
   Repeat
      While DataReady(ComNrI) do begin
         TegnI := ReceivedTegn(ComNrI);
         Gotoxy(Xi,Yi); SkrivTegn(TegnI);
         Getxy(Xi,Yi);
         if Yi > RecMax then begin
            Scroll(1,RecMin,80,RecMax);
            Xi := 1; Yi := RecMax;
         end;
      end;
      if THRemty(ComNrO) then begin
         if SpecTegn <> NUL then begin   (* sidste ikke sendt *)
            SendTegn(ComNrO,SpecTegn);
            Gotoxy(Xo,Yo); SkrivTegn(SpecTegn);
            Getxy(Xo,Yo);
            if Yo > SendMax then begin
               Scroll(1,SendMin,80,SendMax);
               Xo := 1; Yo := SendMax
            end;
            if Eof(InFile) then Stop := True;
            SpecTegn := NUL;
         end
         else begin
            Read(Infile,TegnO);
            if TegnO IN (.EOT,CAN,DLE.) then begin
               SpecTegn := TegnO;
               SendTegn(ComNrO,DLE);
            end
            else begin
               SendTegn(ComNrO,TegnO);
               Gotoxy(Xo,Yo); SkrivTegn(TegnO);
               Getxy(Xo,Yo);
               if Yo > SendMax then begin
                  Scroll(1,SendMin,80,SendMax);
                  Xo := 1; Yo := SendMax
               end;
               if Eof(InFile) then Stop := True;
            end;
         end;
      end;
      FunkTegn := NUL;
      if KeyPressed then begin
         TegnO := ReadKey;
         if TegnO = NUL then FunkTegn := ReadKey;
         if FunkTegn = CtrlF3 then begin
            InitRS232('x',ComNrI,ComNrO);
            TextColor(White); TextBackground(Black);
            FunkTegn := NUL;
         end;
      end;
   Until Stop OR (FunkTegn = CtrlF1);

   Repeat until THRemty(ComNrO);
   if FunkTegn = CtrlF1 then SendTegn(ComNrO,CAN)   (* Cancel *)
   else                      SendTegn(ComNrO,EOT);  (* End Of Transmision *)

   Close(InFile);

   Gotoxy(01,25);
   TextColor(Black); TextBackground(White);

   if TegnO = CtrlF1 then
      Write(BEL,' Filoverfrsel er afbrudt af modtager - tryk p en tast                       ')
   else
      Write(' Filoverfrsel er nu fuldfrt  -  tryk p en tast                             ');

   TextColor(White); TextBackground(Black);
   TegnO := ReadKey;

   if DataReady(ComNrI) then TegnI := ReceivedTegn(ComNrI); (* tm evt. tegn *)
end;


Procedure ReceiveFile(ComNrI,ComNrO: integer);
Const
   Sendmin = 20; SendMax = 24;
   Recmin  =  2; Recmax  = 18;
Var
   TegnI,TegnO: Char;
   Xi,Yi,Xo,Yo: Integer;
   Tegn: char;
   OutFile: File of char;
   OutFileName: String(.64.);
   IOtest: integer;
   SpecTegn:    char;
   Stop:        boolean;
   FunkTegn: char;
   FunkSend: byte;

begin
   ClrScr;
   Repeat
      Write('Indtast Navn p modtager fil: ');ReadLN(OutFileName);
      if OutFileName = '' then EXIT;
      Assign(Outfile,OutFileName);
      (*$I-*) Rewrite(OutFile); (*$I+*)
      IOtest := IOresult;
      if IOtest <> 0 then begin
         WriteLN('Fejl ved bning af file  -  fejlkode ',Hex(Chr(lo(IOtest))));
      end;
   until IOtest = 0;

   TransSkrm(ComNrI,ComNrO,Sendmin,Sendmax,Recmin,Recmax,False);

   if DataReady(ComNrI) then TegnI := ReceivedTegn(ComNrI); (* tm evt. gl. *)

   Xi := 1; Yi := Recmin;
   Xo := 1; Yo := Sendmin;

   Stop := False;
   SpecTegn := NUL;
   TegnI := ' ';
   TegnO := ' ';
   Gotoxy(Xo,Yo);
   Repeat
      While DataReady(ComNrI) do begin
         TegnI := ReceivedTegn(ComNrI);
         if (SpecTegn = NUL) AND (TegnI = DLE) then begin
            SpecTegn := TegnI;
         end
         else begin
            if (SpecTegn = NUL) AND (TegnI IN (.EOT,CAN.)) then begin
               Stop := True;
            end
            else begin
               Gotoxy(Xi,Yi); SkrivTegn(TegnI);
               Getxy(Xi,Yi);
               if Yi > RecMax then begin
                  Scroll(1,RecMin,80,RecMax);
                  Xi := 1; Yi := RecMax;
               end;
               Write(OutFile,TegnI);
               SpecTegn := NUL;
            end;
         end;
      end;
      if TegnO = CR then begin   (* fra sidste afsending *)
         if THRemty(ComNrO) then begin
            SendTegn(ComNrO,LF);
            TegnO := ' ';
         end;
      end
      else if KeyPressed OR (FunkSend > 0) then if THRemty(ComNrO) then begin
         if FunkSend > 0 then begin
            if FunkSend = 2 then
               TegnO := NUL
            else begin
               TegnO    := FunkTegn;
               FunkTegn := NUL;
            end;
            FunkSend := FunkSend - 1;
         end
         else begin
            TegnO := ReadKey;
            if TegnO = NUL then begin
               FunkTegn := ReadKey;
               FunkSend := 2;
            end;
         end;
         if FunkSend <> 2 then begin
            SendTegn(ComNrO,TegnO);
            Gotoxy(Xo,Yo); SkrivTegn(TegnO);
            if TegnO = CR then SkrivTegn(LF);
            Getxy(Xo,Yo);
            if Yo > SendMax then begin
               Scroll(1,SendMin,80,SendMax);
               Xo := 1; Yo := SendMax
            end;
         end;
         if FunkTegn = CtrlF3 then begin
            InitRS232('x',ComNrI,ComNrO);
            TextColor(White); TextBackground(Black);
            FunkTegn := NUL;
            FunkSend := 0;
         end;
      end;

   Until ( (FunkTegn = CtrlF1) AND (SpecTegn = NUL)) OR (Stop);

   Close(OutFile);

   Gotoxy(01,25);
   TextColor(Black); TextBackground(White);
   if Stop then begin
      if TegnI = EOT then
         Write(' Filoverfrsel er nu fuldfrt  - tryk p en tast                              ')
      else
         Write(BEL,' Filoverfrsel er AFBRUDT af afsender  - tryk p en tast                      ');
   end
   else begin
      Repeat until THRemty(ComNrO);
      SendTegn(ComNrO,CAN);   (* Cancel *)
      Write(BEL,' Filoverfrsel er afbrudt af bruger  - tryk p en tast                        ');
   end;
   TextColor(White); TextBackground(Black);
   TegnO := ReadKey;

   if DataReady(ComNrI) then TegnI := ReceivedTegn(ComNrI); (* tm evt. tegn *)
end;


Var
   ComNrI,ComNrO: integer;

begin  (* HOVEDSTYRING *)
   SkrmBufAdr := GetSkrmBufAdr;

   if SkrmBufAdr = $F000 then DirectVideo := False;

   ClrScr;

   OpsaetPortAdr;

   ComNrI := 1; ComNrO := 1;
   if ParamStr(1) = '1' then ComNrI := 1;
   if ParamStr(1) = '2' then ComNrI := 2;
   if ParamStr(2) = '1' then ComNrO := 1;
   if ParamStr(2) = '2' then ComNrO := 2;

   InitPort(ComNrI);
   InitPort(ComNrO);

   Repeat
      GotoXY(55,25);Write('(C) Copyright Bjrk Busch');
      GotoXY(21,01);Write('Asynkron RS232 komunikation');
      GotoXY(21,04);Write('-------- Hovedmenu --------');
      GotoXY(21,07);Write('A: Receive & Send');
      GotoXY(21,08);Write('B: Send file');
      GotoXY(21,09);Write('C: Receive file');
      GotoXY(21,10);Write('P: St/test portflag');
      GotoXY(21,11);Write('R: St hastighed m.m. for modtagelse');
      GotoXY(21,12);Write('S: St hastighed m.m. for afsendelse');
      GotoXY(21,13);Write('T: TTY terminal emulering');
      GotoXY(21,15);Write('X: Stop program');
      GotoXY(21,17);Write('Vlg funktion med bogstav');
      SR := ReadKey;
      Case SR of
          'a','A': ReceiveSend(ComNrI,ComNrO);
          'b','b': SendFile(ComNrI,ComNrO);
          'c','c': ReceiveFile(ComNrI,ComNrO);
          'p','P': TestPort;
          'r','R': InitRS232('r',ComNrI,ComNrO);
          's','S': InitRS232('s',ComNrI,ComNrO);
          't','T': TTYTerminalEmulering(ComNrI,ComNrO);
          else;
      end;
      ClrScr;
   Until SR IN (.'X','x'.);

   ClrScr;
   WriteLN('                                                            ');
   WriteLN('                                                            ');
   WriteLN('                                                            ');
   WriteLN('             CopyRight  Bjrk Boye Busch                    ');
   WriteLN('                                                            ');
   WriteLN('             TietgenSkolen                                  ');
   WriteLN('             EDB Skolen - Odense                            ');

   Delay(1500);
   


end.
